################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
diff_transcript_sets.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python diff_transcript_sets.py --help

Type::

   python diff_transcript_sets.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import sets
import optparse
import math
import tempfile

""" program $Id: diff_transcript_sets.py 2781 2009-09-10 11:33:14Z andreas $

show differences between two transcript sets.
"""
import CGAT.Experiment as E
import CGAT.IOTools as IOTools

def parseIdentifier( id, options ):
    data = id.split(options.separator)

    if len(data) == 4:
        species, transcript, gene = data[:3]
    elif len(data) == 2:
        species, gene = data
        transcript = gene
    else:
        options.stderr.write("# parsing error for '%s'\n" % id )
        raise ValueError
    
    return species, transcript, gene

def countGenesTranscripts( inlist, options ):
    """count number of genes/transcripts in list."""

    genes = {}
    transcripts = {}
    
    for x in inlist:

        try:
            species, transcript, gene = parseIdentifier( x, options )
        except ValueError:
            continue
        
        if species not in genes:
            genes[species] = set()
            transcripts[species] = set()

        transcripts[species].add(transcript)
        genes[species].add(gene)
        
    return genes, transcripts

def getTranscriptsForGenes( genes, transcripts, options ):
    """get transcripts for list of genes."""

    result = []
    for x in transcripts:
        try:
            species, transcript, gene = parseIdentifier( x, options )
        except ValueError:
            continue
        
        if gene in genes:
            result.append( x )
    return result

##--------------------------------------------------------------------------
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: diff_transcript_sets.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-p", "--add-percent", dest="add_percent", action="store_true",
                      help="add percent columns" )
    parser.add_option("-d", "--dump-sets", dest="dump_sets", action="append", type="choice",
                      choices=("rest_genes1", "rest_genes2", "intersection", "union"),
                      help="dump sets of transcripts/genes" )
    parser.add_option("-o", "--output-pattern", dest="output_pattern", type="string",
                      help="output pattern to use for dumped sets. Should contain one %s." )
    
    parser.set_defaults(
        separator = "|",
        add_percent = "False",
        dump_sets = [],
        output_pattern = "%s",
        )

    (options, args) = E.Start( parser, add_psql_options = True )

    options.filename1, options.filename2 = args

    ids1, nerrors1 = IOTools.ReadList( open(options.filename1, "r") )
    ids2, nerrors2  = IOTools.ReadList( open(options.filename2, "r") )

    genes1, transcripts1 = countGenesTranscripts( ids1, options )
    genes2, transcripts2 = countGenesTranscripts( ids2, options )    

    options.stdout.write( "species\tngenes1\tntranscripts1\tngenes2\tntranscripts2\ttr_inter\ttr_union\ttr_rest1\ttr_rest2\ttr_inter\tg_union\tg_rest1\tg_rest2" )
    options.stdout.write( "\ttr_rest1\ttr_rest2\tg_rest1\tg_rest2")
    
    options.stdout.write ("\n" )
    
    for species in set(genes1.keys()).union(set(genes2.keys())):
        nt1, nt2, ng1, ng2 = "na", "na", "na", "na"
        
        if species in genes1:
            g1 = genes1[species]
            t1 = transcripts1[species]
            nt1 = "%i" % len(transcripts1[species])
            ng1 = "%i" % len(genes1[species])
        else:
            t1, g1 = None, None
        
        if species in genes2:
            g2 = genes2[species]
            t2 = transcripts2[species]            
            nt2 = "%i" % len(transcripts2[species])
            ng2 = "%i" % len(genes2[species])
        else:
            t2, g2 = None, None
            
        if species in transcripts1 and transcripts2:
            ct = "%i" % len(t1.intersection(t2))
            ut = "%i" % len(t2.union(t1))
            rt1 = "%i" % len(t1.difference(t2))
            rt2 = "%i" % len(t2.difference(t1))
        else:
            ct, ut, rt1, rt2 = ["na"] * 4

        if species in genes1 and genes2:
            cg = "%i" % len(g1.intersection(g2))
            ug = "%i" % len(g2.union(g1))
            rg1 = "%i" % len(g1.difference(g2))
            rg2 = "%i" % len(g2.difference(g1))
        else:
            cg, ug, rg1, rg2 = ["na"] * 4
            
        options.stdout.write( "\t".join( (species, nt1, ng1, nt2, ng2)) )
        options.stdout.write( "\t" )
        options.stdout.write( "\t".join( (ct, ut, rt1, rt2)) )        
        options.stdout.write( "\t" )        
        options.stdout.write( "\t".join( (cg, ug, rg1, rg2)) )

        if options.add_percent:
            if species in genes1 and genes2:
                rg1 = "%5.2f" % (100.0 * len(g1.difference(g2)) / len(g1))
                rg2 = "%5.2f" % (100.0 * len(g2.difference(g1)) / len(g2))
            if species in transcripts1 and transcripts2:
                rt1 = "%5.2f" % (100.0 * len(t1.difference(t2)) / len(t1))
                rt2 = "%5.2f" % (100.0 * len(t2.difference(t1)) / len(t2))
            options.stdout.write( "\t" )                        
            options.stdout.write( "\t".join( (rt1, rt2, rg1, rg2)) )
        
        options.stdout.write( "\n" )

        for choice in options.dump_sets:

            output_set = None
            
            if choice == "rest_genes1" and g1 and g2:
                output_set = getTranscriptsForGenes(g1.difference(g2), ids1, options )
                
            elif choice == "rest_genes2" and g1 and g2:
                output_set = getTranscriptsForGenes(g2.difference(g1), ids2, options )
                
            if output_set:
                outfile = open( options.output_pattern % (choice), "w" )
                for x in output_set:
                    outfile.write( "%s\n" % (x,) )
                outfile.close()
                    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
SVGDuplicationsWheel.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, getopt, time, optparse, math, tempfile, bisect

USAGE="""python optic/plot_duplications.py [OPTIONS] < stdin > stdout

plots a wheel plot of duplications in SVG format.
The script requirse the following input files:

--contig-sizes: a filename with contig sizes to output. This is a tab-separated
    file with contig<tab>size on each line.

Example for contig sizes:
chr1    1000
chr2    1000
chr3    1000

The duplications are input via standard input as a tab-separated table containing
the following three fields:

   1. cluster_id: an identifier for the cluster
   2. locations: a ';' separated list of duplicated genes with their locations. Each
        entry is of the following format:
        gene_id:chr:strand:start:end
   3. tree: the tree of duplicated genes. This is used to color arcs by tree height.
   The trees are in newick format.

Example for standard input:

1       gene1:chr1:+:100:200;gene2:chr2:+:100:200       ((gene1:0.5,gene2:0.5):0.5);
2       gene3:chr1:+:300:400;gene4:chr3:+:300:400       ((gene3:0.2,gene4:0.2):0.2);

Output: the svg formatted file is writtent to standard output. Comment lines
start with a '#'. You can remove this by setting -v 0 or --verbose=0.
"""

import Experiment
import IOTools
import SVGdraw 
import TreeTools

# some definitions for the layout of the picture
BLACK = (0,0,0)
WHITE = (255,255,255)
RED   = (255,0,0)
GREEN = (0,255,0)
BLUE  = (0,0,255)
YELLOW = (255,255,0)
CYAN   = (0,255,255)
PURPLE = (255,0,255)
GREY = (128,128,128)
ORANGE = (255,125,0)

class DuplicationPlot:

    def __init__(self,
                 contigs,
                 map_contig2size,
                 num_entries,
                 template = "screen",
                 ):

        self.mElements = {}
        
        ## save contig sizes
        self.contigs = contigs
        self.mMapContig2Size = map_contig2size

        ## get row size
        self.mNumEntries = num_entries

        self.mRadius = 3000
        self.mRadiusIncrement = 40
        self.mRadiusStart = self.mRadius

        self.mMinValue = 0.0
        self.mMaxValue = 1.0

        ## planes
        self.mPlaneJoins = 4
        self.mPlaneLinks = 5
        self.mPlaneGrid = 10

        ## set of previous points
        self.mPreviousPoints = set()

        ## whether or not coordinates are forward coordinates
        self.mForwardCoordinates = False

        self.mTemplate = template
        
    def setFormattingOptions( self):
        """set formatting options according to template.

        This function is called, after basic dimensions have been set
        like the size of the panel and radius increments.
        """

        ## a space
        self.mSeparator = 10
        
        ## colours for duplications
        ## coloured by distance
        self.mLinkStrokeWidthSymbol = 1

        ## radial connection (same contig)
        self.mLinkRadStrokeWidth = 10
        ## arc connection (different contigs)
        self.mLinkArcStrokeWidth = 10
        
        self.mLinkStrokeWidthMasked = 1
        self.mLinkColourMasked = GREY
        self.mLinkColourSymbolMasked = GREY
        
        self.startColour  = RED
        self.mMiddleColour = YELLOW
        self.mStopColour   = GREEN
        
        ## size of symbol
        self.mLinkSymbolSize = self.mRadiusIncrement / 4

        ## Parameters for the grid
        self.mGridStrokeWidth = 1            
        self.mGridColour = GREY

        ## maximum size of a box
        self.mMaxBoxSize = 100
        self.mTextSize   = 100

        ## Height and width get set
        self.mHeaderFontSize  = self.mTextSize
        self.mHeaderFont      = "Verdana"

        self.mGridFontSize    = self.mRadius / 10
        self.mGridFont        = "Verdana"

        ## Scale
        self.mScaleFontSize   = self.mRadius / 10
        self.mScaleFont       = "Verdana"
        self.mScaleNumTicks = 20
        self.mScaleBoxSizeY = 30
        
        ## Footer
        self.mFooterFontSize  = self.mTextSize
        self.mFooterFont      = "Verdana"
        self.mFooter = None

        ## Title
        self.mTitleFontSize  = self.mTextSize
        self.mTitleFont      = "Verdana"
        self.mTitle = None

        self.mAngleResolution = 1

        if self.mTemplate == "screen":
            ## screen is default
            pass
        elif self.mTemplate == "publication":
            ## for publication: increase line widths and
            ## symbol size by a factor
            factor = 4
                    
            self.mAngleResolution = 2

            self.mLinkSymbolSize  *= factor
            self.mLinkRadStrokeWidth *= factor
            self.mLinkArcStrokeWidth *= factor            
            self.mGridStrokeWidth = self.mLinkRadStrokeWidth / 2

            self.startColour  = RED
            self.mMiddleColour = ORANGE
            self.mStopColour   = YELLOW

        elif self.mTemplate == "poster":
            ## for publication: increase line widths and
            ## symbol size by a factor
            ## use red -> blue as links
            ## reduce thickness of arcs
            factor = 4
                    
            self.mAngleResolution = 2

            self.mLinkSymbolSize  *= factor
            self.mLinkRadStrokeWidth *= factor / 2
            self.mLinkArcStrokeWidth *= factor / 3
            self.mGridStrokeWidth = self.mLinkRadStrokeWidth / 2

            self.mGridColour   = BLACK
            self.startColour  = RED
            self.mMiddleColour = None
            self.mStopColour   = BLUE

        ## elements within wheel
        self.mWheelElements = []
        
    #####################################################################
    def addElement( self, element, plane = 0):
        """add element to list in plane.
        
        The list is later sorted by priority.
        """
        if plane not in self.mElements:
            self.mElements[plane] = []
        self.mElements[plane].append( element )

    #####################################################################
    def addWheelElement( self, element, plane = 0):
        self.mWheelElements.append( (element, plane ) )
        
    #####################################################################                        
    def initializePlot( self ):
        """set dimensions of the plot.

        Header is left and top, footer is right and bottom.
        """

        ## set formatting options according to user input
        self.setFormattingOptions( )

        ## build coordinates for rows and columns
        self.buildMap2Position()

        ## build colour map
        self.buildColourMap()

        self.mTotalSize = sum( [self.mMapContig2Size[x] for x in self.contigs] )

        ## smallest radius
        self.mRadiusStart = self.mRadius

        ## radius to fall back on
        self.mRadiusFallBack = self.mRadius
        
        self.mDataMiddleX, self.mDataMiddleY = 0, 0

        ## maximum coordinate of longest range for fallback
        self.mLastMax = 0

        ## maximum used radius in slice
        self.mRadiusMax = self.mRadius
        
        ## maximum coordinate of previous range
        self.mPreviousMax = 0
        
        self.mLastChr = None

        ## keeps track of written separators, so that they do not get written multiply
        self.mSeparators = {}

        ## minimum distance for resolving dots
        self.mMinDistance = self.mTotalSize / 360.0 

        ## whether or not last entry was cis
        self.mPreviousCis = False

        ## number format for legend
        self.mFormatNumberLegend = "%5.2f"

        
    #####################################################################
    def setTitle( self, title ):
        """set title."""
        self.mTitle = title

    #####################################################################
    def setFooter( self, footer ):
        """set footer."""
        self.mFooter = footer
        
    #####################################################################
    def writeTitle( self ):
        """write title into plot."""
        
        if self.mTitle:
            
            e = SVGdraw.text( self.mPageWidth / 2,
                              self.mTitleFontSize ,
                              self.mTitle,
                              self.mTitleFontSize,
                              self.mTitleFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle" )

            self.addElement(e)

    #####################################################################        
    def buildColourMap( self ):
        """build map of heights to colours.

        If self.mMiddleColour is defined, the gradient will be built in two steps:
        1: first half: start to middle
        2: second half: middle to end

        Otherwise, only one gradient is built.
        """

        self.mColours = []
        self.mColourThresholds = []
        value = self.mMinValue
        num_intervalls = 200
        
        if self.mMiddleColour:
            num_steps = num_intervalls / 2
            
            self.mColourIncrement = float((self.mMaxValue - self.mMinValue)) / num_steps / 2

            d = map( lambda x, y: (x - y) / num_steps, self.mMiddleColour, self.startColour)
            for x in range(num_steps):
                self.mColours.append ( (self.startColour[0] + x * d[0],
                                        self.startColour[1] + x * d[1],
                                        self.startColour[2] + x * d[2] ) )
                value += self.mColourIncrement
                self.mColourThresholds.append( value )

            self.mColours.append( self.mMiddleColour )
            value += self.mColourIncrement
            self.mColourThresholds.append( value )

            start_colour = self.mMiddleColour
        else:
            num_steps = num_intervalls             
            start_colour = self.startColour
            self.mColourIncrement = float((self.mMaxValue - self.mMinValue)) / num_steps 

        d = map( lambda x, y: (x - y) / num_steps, self.mStopColour, start_colour)
        for x in range(num_steps):
            self.mColours.append ( (start_colour[0] + x * d[0],
                                    start_colour[1] + x * d[1],
                                    start_colour[2] + x * d[2] ) )

            value += self.mColourIncrement
            self.mColourThresholds.append( value )

        self.mColours.append( self.mStopColour )

    #####################################################################        
    def buildMap2Position( self ):
        """build map of contigs to start."""
        self.contig2Position = {}

        l = 0
        for x in self.contigs:
            self.contig2Position[x] = l
            l += self.mMapContig2Size[x]

    #####################################################################
    def addSeparator( self ):
        """add separator on circles."""
        if self.mRadius not in self.mSeparators:
            e = SVGdraw.circle( self.mDataMiddleX, self.mDataMiddleY, self.mRadius, fill="none",
                                stroke = "rgb(%i,%i,%i)" % self.mGridColour,
                                stroke_width = self.mGridStrokeWidth )
            self.addWheelElement( e, self.mPlaneGrid )
        self.mSeparators[self.mRadius] = 1
        
    #####################################################################
    def getPosition( self, chr, strand, residue ):
        """return position on arc where a residue lies."""
        if self.mForwardCoordinates:
            return self.contig2Position[chr] + residue
        else:
            if strand in ("-", 0, "0"):
                return self.contig2Position[chr] + self.mMapContig2Size[chr] - residue
            else:
                return self.contig2Position[chr] + residue            

    #####################################################################
    def getAngle( self, pos ):
        """return angle on arc for position pos."""
        
        return 360.0 * pos / self.mTotalSize

    #####################################################################
    def getPosOnArc( self, angle, radius):
        """return position on arc centered on x, y at angle and radius."""
        
        degPerRad = math.pi / 180.0
        
        dx = radius* math.cos( angle* degPerRad)
        dy = radius* math.sin( angle* degPerRad)
        
        return dx + self.mDataMiddleX, dy + self.mDataMiddleY

    #####################################################################
    def getPosOnArcForRadius( self, x, radius ):
        """get pos on arc for coordinate x."""
        return self.getPosOnArc( self.getAngle(x), radius)

    #####################################################################
    def addDuplication( self, entries, map_gene2pos, height,
                        url = None,
                        with_separator=True,
                        link_to_previous = False,
                        quality2symbol = {},
                        quality2mask = {}):
        """add a dot in row/col."""
        
        mi, ma = None, 0

        pos = bisect.bisect( self.mColourThresholds, height )
        master_colour = self.mColours[pos]
        
        chrs = {}
        points = []

        if not link_to_previous:
            self.mPreviousPoints = {}

        ########################################################
        ########################################################
        ########################################################            
        ## convert gene list to a set of points
        ########################################################            
        for species, transcript, gene, quality in entries:

            chr, strand, first_res, last_res = map_gene2pos[gene]
            chrs[chr] = 1
            pos1 = self.getPosition( chr, strand, first_res )
            pos2 = self.getPosition( chr, strand, last_res )
            
            a = min( pos1, pos2 )
            b = max( pos1, pos2 )

            if mi == None:
                mi = a
            else:
                mi = min(a, mi)
            ma = max(b, ma)
            
            points.append( (pos1, pos2, gene, quality, chr) )

        ########################################################
        ########################################################
        ########################################################            
        ## decide whether we need to increment the radius
        ########################################################                        
        cis = len(chrs) == 1

        old_radius = self.mRadius
        is_overlap = False
        if cis:
            if not self.mLastChr:
                self.mLastChr = chr

            if chr != self.mLastChr:
                self.mRadius = self.mRadiusFallBack
                self.mLastMax = ma
                self.mPreviousMax = ma
                self.mLastChr = chr
            else:
                if self.mPreviousMax + self.mMinDistance > mi:
                    ## overlap due to close proximitiy
                    self.mRadius += self.mRadiusIncrement
                    if with_separator: self.addSeparator()

                    ## true overlap
                    if self.mPreviousMax > mi:
                        is_overlap = True
                    
                elif self.mLastMax + self.mMinDistance > mi:
                    pass
                else:
                    self.mRadius = self.mRadiusFallBack
                    
                self.mLastMax = max(self.mLastMax, ma)
        else:
            if self.mLastMax > mi:
                self.mRadius += self.mRadiusIncrement
                if with_separator: self.addSeparator()

        self.mPreviousMin = mi
        self.mPreviousMax = ma
        self.mPreviousCis = cis
        
        self.mRadiusMax = max(self.mRadius, self.mRadiusMax)

        ########################################################
        ########################################################
        ########################################################            
        ## draw points 
        ########################################################                    
        link_colour = master_colour
        link_rad_width = self.mLinkRadStrokeWidth
        link_arc_width = self.mLinkArcStrokeWidth
        new_points = {}
        for pos1, pos2, gene, quality, chr in points:

            angle = self.getAngle( (pos1 + pos2) / 2 )
            
            x,y = self.getPosOnArc( angle, self.mRadius )

            try:
                symbol = quality2symbol[quality]
            except KeyError:
                symbol = "rect"

            if quality in quality2mask:
                colour = self.mLinkColourSymbolMasked
                link_colour = self.mLinkColourMasked
                link_rad_width = self.mLinkStrokeWidthMasked
                link_arc_width = self.mLinkStrokeWidthMasked
            else:
                colour = master_colour

            if gene in self.mPreviousPoints:
                continue
            
            new_points[gene] = (x, y, angle, quality, chr)
            
            if symbol == "circle":
                ee = SVGdraw.circle( x, y, self.mLinkSymbolSize,
                                     fill = "rgb(%i,%i,%i)" % colour,
                                     stroke="black",
                                     stroke_width = self.mLinkStrokeWidthSymbol )
            elif symbol == "rect":
                ee =  SVGdraw.rect( x-self.mLinkSymbolSize/2, y-self.mLinkSymbolSize/2,
                                    self.mLinkSymbolSize, self.mLinkSymbolSize,
                                    fill = "rgb(%i,%i,%i)" % colour,
                                    stroke="black",
                                    stroke_width = self.mLinkStrokeWidthSymbol )

            if url:
                e = SVGdraw.link( url % gene )
                e.addElement( ee )
            else:
                e = ee
                
            self.addWheelElement( e )                
            
        ########################################################
        ########################################################
        ########################################################
        ## write all arcs in between old points and new points
        ## cis:   circular arc
        ## trans: radial arc
        ########################################################   

        angles = []
        
        for x1,y1,angle1,quality1,chr1 in new_points.values():

            ## reduce clutter by not writing arc to the same angle
            for x2,y2,angle2,quality2,chr2 in self.mPreviousPoints.values():

                for a in angles:
                    if a - self.mAngleResolution < angle2 < a + self.mAngleResolution:
                        break
                else:
                    angles.append( angle2 )

                    d = SVGdraw.pathdata( x1, y1 )

                    if chr1 == chr2:
                        d.relellarc( self.mRadius, self.mRadius, 0, 0, 1, x2-x1, y2-y1 )
                        link_width = link_rad_width
                    else:
                        d.relellarc( self.mRadius * 2, self.mRadius * 2, 0, 0, 0, x2-x1, y2-y1 )
                        link_width = link_arc_width
                        
                    e = SVGdraw.path( d,
                                      fill = "none",
                                      stroke = "rgb(%i,%i,%i)" % link_colour,
                                      stroke_width = link_width )

                    self.addWheelElement(e, self.mPlaneLinks)


        ## plot lines between new points
        new_genes = new_points.keys()

        for g1 in range(len(new_genes)-1):
            
            x1,y1,angle1,quality1,chr1 = new_points[new_genes[g1]]
            
            for g2 in range(g1+1, len(new_genes)):
                
                x2,y2,angle2,quality2,chr2 = new_points[new_genes[g2]]

                for a in angles:
                    if a - self.mAngleResolution < angle2 < a + self.mAngleResolution:
                        break
                else:
                    angles.append( angle2 )

                    d = SVGdraw.pathdata( x1, y1 )

                    if chr1 == chr2:
                        d.relellarc( self.mRadius, self.mRadius, 0, 0, 1, x2-x1, y2-y1 )
                        link_width = link_rad_width                        
                    else:
                        d.relellarc( self.mRadius * 2, self.mRadius * 2, 0, 0, 0, x2-x1, y2-y1 )
                        link_width = link_arc_width                        

                    e = SVGdraw.path( d,
                                      fill = "none",
                                      stroke = "rgb(%i,%i,%i)" % link_colour,
                                      stroke_width = link_width )

                    self.addWheelElement(e, self.mPlaneLinks)
                
        ## add new points to old points
        for k, v in new_points.items():
            self.mPreviousPoints[k] = v

    #####################################################################
    def pushRadius( self ):
        """push radius as fallback."""
        
        self.mRadiusFallBack = self.mRadiusMax + self.mRadiusIncrement
        self.mRadius = self.mRadiusFallBack
        # self.addSeparator()
        self.mLastMax = 0
        self.mPreviousMax = 0
        
    #####################################################################
    def writeGrid( self ):
        """add grid lines."""

        middlex = self.mDataMiddleX
        middley = self.mDataMiddleY

        ## print separators
        for c in range(len(self.contigs)):

            contig = self.contigs[c]
            
            pos = self.getPosition( contig, "+", 0)
            angle = self.getAngle( pos )

            x,y = self.getPosOnArc( angle, self.mRadius )
            
            e = SVGdraw.line( middlex,
                              middley,
                              x, 
                              y,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              stroke_width = self.mGridStrokeWidth )

            self.addElement(e, self.mPlaneGrid )

            if c < len(self.contigs) - 1:
                next_angle = self.getAngle(self.getPosition( self.contigs[c+1], "+", 0 ) )
            else:
                next_angle = 360
            
            x,y = self.getPosOnArc( angle + float(next_angle - angle) / 2, self.mRadiusStart / 2)
            
            e = SVGdraw.text( x,
                              y,
                              contig,
                              self.mGridFontSize,
                              self.mGridFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "start" )
            # do not rotate text:
            # transform="rotate(%i,%i,%i)" % ( angle, x, y ))

            self.addElement(e)

    #####################################################################
    def writeScale( self ):
        """write scales."""

        current_x = self.mScaleX
        current_y = self.mScaleY + self.mScaleHeight

        nboxes = len(self.mColourThresholds)
        # box size for legend in x-direction
        # subtract size of right-most axis label so that it takes the
        # same width as self.mDataWidth.
        box_size_x = math.ceil( (self.mDataWidth -
                                (self.mScaleFontSize * len(self.mFormatNumberLegend % self.mColourThresholds[-1])))
                                / nboxes )

        # change font size such that it labels will fit between tick-marks
        self.mScaleFontSize = min( self.mScaleFontSize,
                                   (box_size_x * self.mScaleNumTicks * 1.5) / len(self.mFormatNumberLegend % self.mColourThresholds[-1]) )
        
        for x in range(nboxes):

            e = SVGdraw.rect( current_x,
                              current_y,
                              box_size_x,
                              self.mScaleBoxSizeY,                                   
                              fill = "rgb(%i,%i,%i)" % self.mColours[x],
                              stroke = "rgb(%i,%i,%i)" % self.mColours[x])                                   
            
            self.addElement(e)
            
            if x % self.mScaleNumTicks == 0:

                e = SVGdraw.line( current_x,
                                  current_y,
                                  current_x,
                                  current_y + self.mScaleBoxSizeY,
                                  stroke = "rgb(%i,%i,%i)" % BLACK,
                                  stroke_width = 5)
                self.addElement(e)
                
                e = SVGdraw.text( current_x,
                                  current_y - self.mScaleBoxSizeY,
                                  self.mFormatNumberLegend % self.mColourThresholds[x],
                                  self.mScaleFontSize,
                                  self.mScaleFont,
                                  stroke = "rgb(%i,%i,%i)" % BLACK,
                                  text_anchor = "start")
                self.addElement(e)
            
            
            current_x += box_size_x

        
    #####################################################################
    def writeFooter( self ):
        """write footer."""
        
        current_x = self.mFooterX
        current_y = self.mFooterY

        if self.mFooter:

            current_y += max( self.mFooterFontSize, self.mMaxBoxSize) + self.mSeparator
            
            e = SVGdraw.text( self.mPageWidth / 2,
                              current_y + self.mFooterFontSize,
                              self.mFooter,
                              self.mFooterFontSize,
                              self.mFooterFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle")
            
            self.addElement(e)

    #####################################################################
    def writeWheel( self ):
        """write duplications.

        shifts everything according to final canvas size.
        There must a way to do this in SVG using relative
        coordinates.
        """
        
        def shiftElement( element, shiftx, shifty):
            if 'x' in element.attributes:
                element.attributes['x'] += shiftx
            if 'y' in element.attributes:
                element.attributes['y'] += shifty
            if 'cx' in element.attributes:
                element.attributes['cx'] += shiftx
            if 'cy' in element.attributes:
                element.attributes['cy'] += shifty
            if 'x1' in element.attributes:
                element.attributes['x1'] += shiftx
            if 'y1' in element.attributes:
                element.attributes['y1'] += shiftx
            if 'd' in element.attributes:
                ## assumes that path starts with a move
                path = element.attributes['d'].split(" ")
                if path[0] != "M":
                    raise "don't know what to do with paths not starting with 'M': %s" % (" ".join(path))
                path[1] = str(float(path[1]) + shiftx)
                path[2] = str(float(path[2]) + shifty)
                element.attributes['d'] = " ".join(path)
                
            for e in element.elements:
                shiftElement( e, shiftx, shifty )

        shiftx = self.mDataX + self.mDataWidth / 2
        shifty = self.mDataY + self.mDataHeight / 2
        
        for element, plane in self.mWheelElements:

            shiftElement( element, shiftx, shifty )
            self.addElement( element, plane )
        
    #####################################################################        
    def finalizePlot( self ):
        """write remaining parts of the plot."""

        ## add last circle
        self.mRadius = self.mRadiusMax + self.mRadiusIncrement
        self.addSeparator()

        ## From user input, build canvas size
        ## height width of various sections
        self.mHeaderWidth, self.mHeaderHeight = 0, 0
        self.mFooterWidth, self.mFooterHeight = 0, 0
        self.mDataWidth, self.mDataHeight = 0, 0

        if self.mTitle:
            self.mHeaderHeight += self.mSeparator + self.mTitleFontSize
        
        ## corrected height and width of data section
        self.mDataX = self.mHeaderWidth
        self.mDataY = self.mHeaderHeight
        self.mDataWidth  = 2 * self.mRadiusMax + 2 * self.mSeparator
        self.mDataHeight = self.mDataWidth
        self.mDataMiddleX = self.mDataX + self.mDataWidth / 2
        self.mDataMiddleY = self.mDataY + self.mDataHeight / 2

        ## scale
        self.mScaleX = self.mHeaderWidth + self.mSeparator
        self.mScaleY = self.mHeaderHeight + self.mSeparator + self.mDataHeight + self.mSeparator
        self.mScaleHeight =  2 * max( self.mScaleFontSize, self.mMaxBoxSize) + self.mSeparator
        self.mScaleWidth = 0

        ## height of footer 
        if self.mFooter:
            self.mFooterHeight = 2 * self.mFooterFontSize
        else:
            self.mFooterHeight = 0

        self.mFooterX = self.mHeaderWidth + self.mDataWidth + self.mScaleWidth + self.mFooterWidth + 3 * self.mSeparator
        self.mFooterY = self.mHeaderHeight + self.mDataHeight + self.mScaleHeight + self.mFooterHeight + 3 * self.mSeparator 

        ## compute actual page width and height
        self.mPageWidth = self.mHeaderWidth + self.mDataWidth + self.mScaleWidth + self.mFooterWidth + 3 * self.mSeparator
        self.mPageHeight = self.mHeaderHeight + self.mDataHeight + self.mScaleHeight + self.mFooterHeight + 3 * self.mSeparator        

        self.writeWheel()
        self.writeGrid()
        self.writeTitle()
        self.writeScale()
        self.writeFooter()

    #####################################################################        
    def writeToFile(self, outfile):
        """write svg image to file.
        """
        self.finalizePlot()

        self.mRoot = SVGdraw.drawing()
        self.mDraw = SVGdraw.svg( (0, 0, self.mPageWidth, self.mPageHeight ) , "100%", "100%" )

        kk = self.mElements.keys()
        kk.sort()
        kk.reverse()
        for k in kk:
            for e in self.mElements[k]:
                self.mDraw.addElement( e )
            
        self.mRoot.setSVG(self.mDraw)

        tfile = tempfile.mktemp()
        
        self.mRoot.toXml( tfile )

        lines = open(tfile,"r").readlines()
        
        outfile.write(string.join(lines,""))
        outfile.write("\n")
        
        os.remove(tfile)

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: SVGDuplicationsWheel.py 2784 2009-09-10 11:41:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-e", "--headers", dest="headers", action="store_true",
                      help="first row is a header [ignored]."  )
    parser.add_option("-t", "--title", dest="title", type="string",
                      help="page title.")
    parser.add_option("-f", "--footer", dest="footer", type="string",
                      help="page footer.")
    parser.add_option("-c", "--contig-sizes", dest="filename_contig_sizes", type="string",
                      help="filname with contig sizes." )
    parser.add_option("-r", "--radius", dest="radius", type="int",
                      help="radius.")
    parser.add_option("-i", "--increment", dest="radius_increment", type="int",
                      help="radius increment.")
    parser.add_option("-u", "--url", dest="url", type="string",
                      help="string to build url for annotation.")
    parser.add_option( "--min-contig", dest="min_contig_size", type="string",
                      help="minimum contig size to delineate.")
    parser.add_option( "--min-value", dest="min_value", type="float",
                       help="minimum branch length.")
    
    parser.add_option( "--max-value", dest="max_value", type="float",
                       help="maximum branch length.")
    
    parser.set_defaults(
        filename_contig_sizes = None,
        headers = False,
        titles = "",
        pattern_filename = None,
        title = "",
        footer = "",
        radius = 3000,
        min_value = 0.0,
        max_value = 0.2,
        url=None,
        radius_increment = 40,
        min_contig_size = 10000,
        remove_empty_contigs = True,
        separator = "|",
        quality2symbol = { 'CG' : "circle", 'PG' : "circle", 'SG' : "circle" },
        quality2mask = ("RG", "CP", "PP", "SP", "RP", "CF", "PF", "SF", "UG", "UP", "UF", "BF", "UK" ),
        sort_by_size = True,
        input_format = "pairwise",
        )

    (options, args) = Experiment.Start( parser, add_pipe_options = True )

    if options.filename_contig_sizes:
        map_contig2size = IOTools.ReadMap( open(options.filename_contig_sizes, "r"),
                                           map_functions=(str, int) )

    ## read data and get contigs that are used (i.e.: remove empty contigs)
    chrs = {}
    lines = sys.stdin.readlines()

    if options.remove_empty_contigs:
        for line in lines:
            if line[0] == "#": continue

            d = line[:-1].split( "\t" )

            cluster_id, in_locations, in_tree = d[:3]

            for l in in_locations.split(";"):
                gene_id, chr, strand, sbjct_from, sbjct_to = l.split(":")
                if chr not in map_contig2size: continue
                chrs[chr] = 1
        for k in map_contig2size.keys():
            if k not in chrs:
                del map_contig2size[k]

    k = map_contig2size.keys()
    
    if len(k) == 0:
        Experiment.Stop()
        sys.exit(0)
    
    k.sort()
    
    if options.sort_by_size:
        k.sort( lambda x,y: cmp(map_contig2size[x], map_contig2size[y]))
        
    plot = DuplicationPlot( k,
                            map_contig2size,
                            num_entries = 0 )

    plot.mRadiusIncrement = options.radius_increment
    plot.mRadius = options.radius
    plot.mMaxValue = options.max_value
    plot.mMinValue = options.min_value
    
    if options.title: plot.setTitle(options.title)
    if options.footer: plot.setFooter(options.footer)    

    plot.initializePlot()
    
    data = []

    if options.input_format == "pairwise":
        
        ## read data from pairwise analysis
        ## format is: cluster_id, locations of duplications, tree of duplications
    
        for line in lines:
            if line[0] == "#": continue

            d = line[:-1].split( "\t" )

            cluster_id, in_locations, in_tree = d[:3]

            mi, ma = 0,0
            found = False
            n = 0
            chrs = {}
            for l in in_locations.split(";"):
                gene_id, chr, strand, sbjct_from, sbjct_to = l.split(":")
                if chr not in map_contig2size: continue
                chrs[chr] = 1
                sbjct_from, sbjct_to = int(sbjct_from), int(sbjct_to)

                xi = plot.getPosition( chr, strand, sbjct_from)
                xa = plot.getPosition( chr, strand, sbjct_to)

                if not mi:
                    mi = xi
                else:
                    mi = min(mi, xi)

                n += 1
                ma = max(ma, xa )
                found = True

            if not found: continue
            cis = len(chrs) == 1
            data.append( (cis, n, mi, ma, cluster_id, in_locations, in_tree) )
        
    data.sort()

    plot.mNumEntries = len(data)
    plot.initializePlot()
    
    last_ndups = 0

    for cis, ndups, mi, ma, cluster_id, in_locations, in_tree in data[:]:

        if ndups != last_ndups:
            plot.pushRadius()
            plot.addSeparator()
            
        last_ndups = ndups
        
        map_gene2location = {}
        for l in in_locations.split(";"):
            gene_id, chr, strand, sbjct_from, sbjct_to = l.split(":")
            if chr not in map_contig2size:
                continue
            sbjct_from, sbjct_to = int(sbjct_from), int(sbjct_to)
            map_gene2location[gene_id] = (chr, strand, sbjct_from, sbjct_to)

        if not map_gene2location: continue
        
        tree = TreeTools.Newick2Tree( in_tree )

        ## the last subset is all nodes again.
        s = TreeTools.GetSubsets( tree )
        is_first = True
        for children, height, branchlength in s[:-1]:
            if len(children)==1: continue
            data = []
            for c in map(lambda x: x.split(options.separator), children):
                if len(c) == 2:
                    data.append( (c[0], c[1], c[1], "CG" ) )
                elif len(c) == 1:
                    data.append( ("unk", c[0], c[0], "CG" ) )
                elif len(c) == 3:
                    data.append( (c[0], c[2], c[3], "CG" ) )

            for species, transcript, gene, quality in data:
                if not gene in  map_gene2location:
                    print >>sys.stderr, in_tree + "\n" +  in_locations + "\n", map_gene2location
                

            plot.addDuplication( data, map_gene2location, height,
                                 url=options.url,
                                 with_separator=is_first,
                                 link_to_previous=not is_first,
                                 quality2symbol = options.quality2symbol,
                                 quality2mask = options.quality2mask)
            is_first = False

    plot.writeToFile(sys.stdout)
    
    Experiment.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperSPP.py - wrap spp peak caller
========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Runs the spp peak caller.

The workflow follows the tutorial at:

http://compbio.med.harvard.edu/Supplements/ChIP-seq/tutorial.html

Usage
-----

Documentation
-------------

Code
----

'''

import os, sys, re, optparse, tempfile, shutil, subprocess
import collections

import Experiment as E
import IOTools

## for zinba
from rpy2.robjects import r as R
import rpy2.robjects as ro
import rpy2.robjects.vectors as rovectors
from rpy2.rinterface import RRuntimeError

def bamToBed( infile, outfile ):
    '''convert bam to bed with bedtools.'''

    statement = "bamToBed -i %(infile)s > %(outfile)s" % locals()

    E.debug( "executing statement '%s'" % statement )

    retcode = subprocess.call(  statement,
                                cwd = os.getcwd(), 
                                shell = True )
    if retcode < 0:
        raise OSError( "Child was terminated by signal %i: \n%s\n" % (-retcode, statement ))

    return outfile

ZinbaPeak = collections.namedtuple( "ZinbaPeak", "contig unrefined_start unrefined_end strand posterior summit height refined_start refined_end median fdr" )

def iteratePeaks( infile ):
    '''iterate of zinba peaks in infile.'''
    
    for line in infile:

        if line.startswith("#"): continue
        if line.startswith("PEAKID\tChrom"): continue
        # skip empty lines
        if line.startswith("\n"): continue

        data = line[:-1].split("\t")

        if len(data) != 12:
            raise ValueError( "could not parse line %s" % line )

        # I assume these are 1-based coordinates
        data[2] = max(int(data[2]) - 1, 0)
        # end
        data[3] = int(data[3])
        # posterior
        data[5] = float(data[5])
        # summit
        data[6] = max(int(data[6]) - 1, 0)
        # height
        data[7] = int(data[7])
        # refined_start
        data[8] = max(int(data[8]) - 1, 0)
        # end
        data[9] = int(data[9])
        # median
        data[10] = int(data[10])
        # qvalue
        data[11] = float(data[11])

        yield ZinbaPeak._make( data[1:] )

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-f", "--input-format", dest="input_format", type="choice",
                      choices = ("bam",),
                      help="input file format [default=%default]."  )
    
    parser.add_option("-w", "--window-size", dest="window_size", type="int",
                      help="window size [default=%default]."  )

    parser.add_option("-c", "--control-filename", dest="control_filename", type="string",
                      help="filename of input/control data in bed format [default=%default]."  )

    parser.add_option("-t", "--threads", dest="threads", type="int",
                      help="number of threads to use [default=%default]."  )

    parser.add_option("-q", "--fdr-threshold", dest="fdr_threshold", type="float",
                      help="fdr threshold [default=%default]."  )
 
    parser.add_option("-z", "--z-threshold", dest="z_threshold", type="float",
                      help="z threshold [default=%default]."  )
 
    parser.add_option( "--bin", dest="bin", type="int",
                       help="bin tags within the specified number of basepairs to speed up calculation;"
                       " increasing bin size decreases the accuracy of the determined parameters [default=%default]")

    parser.add_option( "--srange-min", dest="srange_min", type="float",
                       help = "srange gives the possible range for the size of the protected region;"
                               " srange should be higher than tag length; making the upper boundary too high"
                               " will increase calculation time [%default]" )

    parser.add_option( "--srange-max", dest="srange_max", type="float",
                       help = "srange gives the possible range for the size of the protected region;"
                               " srange should be higher than tag length; making the upper boundary too high"
                               " will increase calculation time [%default]" )
    
    parser.set_defaults(
        input_format = "bam",
        threads = 1,
        fdr_threshold = 0.05,
        window_size = 1000,
        offset = 125,
        srange_min=50,
        srange_max=500,
        bin=5,
        z_threshold = 3,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) != 2:
        raise ValueError("please specify a filename with sample data and an output file")

    filename_sample, filename_output = args[0], args[1]
    filename_control = options.control_filename
    
    # load Zinba
    R.library( 'spp' )
    R.library( 'snow' )
    
    # read data
    E.info( "reading data" )
    R('''chip.data <- read.bam.tags('%s')''' % filename_sample)
    R('''input.data <- read.bam.tags('%s')''' % filename_control)
    R('''cluster = makeCluster( %i )''' % (options.threads) )

    E.info( "computing binding characteristics" )
    # get binding info from cross-correlation profile

    # srange gives the possible range for the size of the protected region;
    # srange should be higher than tag length; making the upper boundary too high will increase calculation time

    # bin - bin tags within the specified number of basepairs to speed up calculation;
    # increasing bin size decreases the accuracy of the determined parameters
    srange_min, srange_max = options.srange_min, options.srange_max
    bin = options.bin
    R('''binding.characteristics <- get.binding.characteristics(chip.data,
                                          srange=c(%(srange_min)i,%(srange_max)i),
                                          bin=%(bin)s,
                                          cluster=cluster);''' % locals())
    # print out binding peak separation distance
    options.stdout.write( "shift\t%i\n" % R('''binding.characteristics$peak$x''')[0])

    ##################################################
    ##################################################
    ##################################################
    E.info( "plot cross correlation profile" )
    # plot cross-correlation profile
    R('''pdf(file="%s.crosscorrelation.pdf",width=5,height=5)''' % filename_output )
    R('''par(mar = c(3.5,3.5,1.0,0.5), mgp = c(2,0.65,0), cex = 0.8);''')
    R('''plot(binding.characteristics$cross.correlation,type='l',xlab="strand shift",ylab="cross-correlation");''')
    R('''abline(v=binding.characteristics$peak$x,lty=2,col=2)''')
    R('''dev.off();''')

    E.info( "selecting informative tags based on the binding characteristics" )
    # select informative tags based on the binding characteristics
    R('''chip.data <- select.informative.tags(chip.data,binding.characteristics);''')
    R('''input.data <- select.informative.tags(input.data,binding.characteristics);''')

    E.info( "outputting broad peaks" )
    window_size, z_threshold = options.window_size, options.z_threshold
    R('''broad.clusters <- get.broad.enrichment.clusters(chip.data,input.data,
                                           window.size=%(window_size)i,
                                           z.thr=%(z_threshold)f,
                                           tag.shift=round(binding.characteristics$peak$x/2))''' % locals())
    # write out in broadPeak format
    R('''write.broadpeak.info(broad.clusters,"%s.broadpeak.txt")''' % filename_output )

    # binding detection parameters
    # desired FDR (1%). Alternatively, an E-value can be supplied to the method calls below instead of the fdr parameter
    # the binding.characteristics contains the optimized half-size for binding detection window
    R('''detection.window.halfsize <- binding.characteristics$whs;''')

    # determine binding positions using wtd method
    E.info( "determining binding positions using wtd method" )
    fdr = options.fdr_threshold
    R('''bp <- find.binding.positions(signal.data=chip.data,control.data=input.data,
                            fdr=%(fdr)f,whs=detection.window.halfsize,cluster=cluster)''' % locals())
    options.stdout.write( "detected_peaks\t%i\n" % R('''sum(unlist(lapply(bp$npl,function(d) length(d$x))))''')[0])

    # output detected binding positions
    R('''output.binding.results(bp,"%s.summit.txt");''' % filename_output)

    R('''bp <- add.broad.peak.regions(chip.data,input.data,bp,
                        window.size=%(window_size)i,z.thr=%(z_threshold)f)''' % locals())
    # output using narrowPeak format
    R('''write.narrowpeak.binding(bp,"%s.narrowpeak.txt")''' % filename_output )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
mali2summary.py - compute summary stats on a mali
=================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

output summary information on a multiple alignment.

* column/row occupancy

Usage
-----

Example::

   python mali2summary.py --help

Type::

   python mali2summary.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Mali as Mali
import scipy
import scipy.stats

def analyzeMali( mali, options, prefix_row = "" ):
    
    if len(mali) == 0:
        raise "not analyzing empty multiple alignment"

    ## count empty sequences
    row_data = map( lambda x: Mali.MaliData( x.mString, options.gap_chars, options.mask_chars ), mali.values() )
    col_data = map( lambda x: Mali.MaliData( x, options.gap_chars, options.mask_chars ), mali.getColumns () )

    if len(row_data) == 0 or len(col_data) == 0: return False
    
    if options.loglevel >= 2:
        for row in row_data:
            options.stdlog.write("# row: %s\n" % str(row) )
        for col in col_data:
            options.stdlog.write("# col: %s\n" % str(col) )

    options.stdout.write(prefix_row)

    ## calculate average column occupancy
    col_mean = scipy.mean( map( lambda x: x.mNChars, col_data ) )
    col_median = scipy.median( map( lambda x: x.mNChars, col_data ) )        
    length = mali.getLength()

    if float(int(col_median)) == col_median:
        options.stdout.write( "%5.2f\t%5.2f\t%i\t%5.2f" % (col_mean, 100.0 * col_mean / length,
                                                              col_median, 100.0 * col_median / length))
    else:
        options.stdout.write( "%5.2f\t%5.2f\t%5.1f\t%5.2f" % (col_mean, 100.0 * col_mean / length,
                                                              col_median, 100.0 * col_median / length))
        
    row_mean = scipy.mean( map( lambda x: x.mNChars, row_data ) )
    row_median = scipy.median( map( lambda x: x.mNChars, row_data ) )        
    width = mali.getWidth()

    if float(int(row_median)) == row_median:
        options.stdout.write( "\t%5.2f\t%5.2f\t%i\t%5.2f" % (row_mean, 100.0 * row_mean / width,
                                                             row_median, 100.0 * row_median / width))
    else:
        options.stdout.write( "\t%5.2f\t%5.2f\t%5.1f\t%5.2f" % (row_mean, 100.0 * row_mean / width,
                                                                row_median, 100.0 * row_median / width))
        
    options.stdout.write("\n")
    
    return True

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: mali2summary.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-i", "--input-format", dest="input_format", type="choice",
                      choices=("plain", "fasta", "clustal", "stockholm" ),
                      help="input format of multiple alignment"  )
    
    parser.add_option("-a", "--alphabet", dest="alphabet", type="choice",
                      choices=("aa", "na"),
                      help="alphabet to use [default=%default].", )

    parser.add_option("-p", "--pattern-mali", dest="pattern_mali", type="string",
                      help="filename pattern for input multiple alignment files."  )

    parser.set_defaults(
        input_format="fasta",
        output_format="fasta",
        mask_chars = "nN",
        gap_chars = "-.",
        alphabet="na",
        pattern_mali = None,
        )

    (options, args) = E.Start( parser )

    if options.pattern_mali:
        prefix_header = "prefix\t"
        prefix_row = "\t"
    else:
        prefix_header = ""
        prefix_row = ""
        
    options.stdout.write( "%sncol_mean\tpcol_mean\tncol_median\tpcol_median\tnrow_mean\tprow_mean\tnrow_median\tprow_median\n" % (prefix_header, ) )

    ninput, nskipped, noutput, nempty = 0, 0, 0, 0

    if options.pattern_mali:

        ids, errors = IOTools.ReadList( sys.stdin )

        E.debug( "read %i identifiers.\n" % len(ids))

        nsubstitutions=len(re.findall("%s", options.pattern_mali))
            
        for id in ids:

            filename = options.pattern_mali % tuple([id] * nsubstitutions)
            ninput += 1
        
            if not os.path.exists( filename ):
                nskipped += 1
                continue
            
            ## read multiple alignment in various formats
            mali = Mali.Mali()
            mali.readFromFile( open(filename, "r"), format = options.input_format )

            if mali.isEmpty():
                nempty += 1
                continue

            E.debug( "read mali with %i entries from %s.\n" % (len(mali), filename))

                
            if analyzeMali( mali, options, prefix_row = "%s\t" % id ):
                noutput += 1
            
    else:
        
        ## read multiple alignment in various formats
        mali = Mali.Mali()
        mali.readFromFile( sys.stdin, format = options.input_format )
        ninput += 1

        if mali.isEmpty():
            nempty += 1
        else:
            E.debug( "read mali with %i entries." % (len(mali)))

            if analyzeMali( mali, options, prefix_row = "" ):
                noutput += 1

    E.info( "ninput=%i, noutput=%i, nskipped=%i, nempty=%i." % (ninput, noutput, nskipped, nempty))
        
    E.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
coverage2stats.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

The output from bedtools genomecov is converted to a set of statistics
for each contig that is output. Statistics output are the mean and sd
of coverage for each contig.

Usage
-----

Example::

   python coverage2stats.py --help

Type::

   python coverage2stats.py --help

for command line help.

Documentation
-------------

The script assumes the input is from stdin and outputs the results to stdout.
The input is expected to be in tab delimited text format (no header line)

          <contig><base-position><coverage>

Code
----

'''

import os
import sys
import re
import optparse
import numpy as np
import collections
import itertools

import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser( version = "%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    inf = options.stdin
    
    coverage_result = collections.defaultdict(list)
    E.info("reading in coverage data")
    for line in inf.readlines():
        data = line[:-1].split("\t")
        contig, coverage = data[0], data[2]
        coverage_result[contig].append(coverage)
    E.info("read %i contigs" % len(coverage_result.keys()))
        
    options.stdout.write("contig\tcov_mean\tcov_sd\n")
    for contig, coverage in coverage_result.iteritems():
        coverage = map(int, coverage)
        options.stdout.write("%s\t%s\t%s\n" % (contig, str(np.mean(coverage)), str(np.std(coverage))))

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
GraphTools.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, time, tempfile, os, subprocess

class Error(Exception):
    """Base class for exceptions in this module."""
    pass

    def __str__(self):
        return str(self.message)

class InputError(Error):
    """Exception raised for errors in the input.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class RuntimeError(Error):
    """Exception raised for errors in the input.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class ExternalGraph:

    mExecutableComponents = "ga_components"

    def __init__( self, filename = None):

        if filename:
           self.mInputFilename = filename
           self.mKeepInput = True
           self.mIsOpen = False
        else:
            x, self.mInputFilename = tempfile.mkstemp()
            os.close(x)
            self.mInputFile = open(self.mInputFilename, "w" )
            self.mIsOpen = True
            self.mKeepInput = False

    def __del__(self):
        
        if not self.mKeepInput:
            if os.path.exists( self.mInputFilename):
                os.remove( self.mInputFilename)
                
    def add_edge( self, node1, node2, weight = None ):
        """add edge to graph.
        """

        if self.mIsOpen:
            if weight != None:
                self.mInputFile.write( "%s\t%s\t%s\n" % (map(str,(node1,node2, weight))))
            else:
                self.mInputFile.write( "%s\t%s\n" % (str(node1),str(node2)))
        else:
            raise InputError( "Graph has already been finalized.")

    def finalize( self ):
        """
        """
        if self.mIsOpen:
            self.mInputFile.close()
            self.mIsOpen = False
            
    def connected_components( self ):
        """calculate connected components for graph."""

        self.finalize()        

        statement = "%s %s " % (self.mExecutableComponents, self.mInputFilename)
        
        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              close_fds = True)                              

        (out, err) = s.communicate()


        if s.returncode != 0:
            raise RuntimeError("Error %s while executing statement\n%s" % (err, statement))

        keep = False
        
        map_cluster2members = {}

        rx2 = re.compile("# total number of components")
        rx1 = re.compile("# token\tvertex")
        for line in out.split("\n"):
            if rx1.match(line):
                keep = 1
            elif rx2.match(line):
                break
            elif keep:
                id, cluster = line.split("\t")[:2]
                if cluster not in map_cluster2members:
                    map_cluster2members[cluster] = []
                map_cluster2members[cluster].append(id)

        return map_cluster2members.values()
                
        
        














####
####
##
## Project PairsDBTools
##
## Copyright (C) 2002 Andreas Heger All rights reserved
##
## Author: Andreas Heger <heger@ebi.ac.uk>
##
## $Id: data2phylocontrasts.py 2782 2009-09-10 11:40:29Z andreas $
##
##
####
####
"""
data2phylocontrasts.py - compute phylogenetic independent contrasts
==============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a tab-separated table and computes phylogenetic independent 
contrasts according to Felsenstein (1985).

In addition to the table, this script requires a tree with branch lengths.
For each taxon in the tree, the tab-separated table should contain a
column.

This script uses :file:`CONTRAST` of the :file:`PHYLIP` package as a back-end.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 

import sys
import re
import string
import os
import optparse
import math

import CGAT.Experiment as E
import scipy
import scipy.stats
import CGAT.WrapperPhylip as WrapperPhylip
import CGAT.TreeTools as TreeTools

def calculateCorrelationCoefficient( a, b):
    """calculate correlation coefficient for regression through the origin."""

    s1 = 0
    s2 = 0
    s3 = 0
    for i in range(len(a)):
        s1 += a[i] * b[i]
        s2 += a[i] * a[i]
        s3 += b[i] * b[i]
    return s1 / math.sqrt( abs(s2) * abs(s3) )

##---------------------------------------------------------------------------------------------------------        
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: data2phylocontrasts.py 2782 2009-09-10 11:40:29Z andreas $",
                                    usage = globals()["__doc__"] )

    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to take for calculating histograms." )
    parser.add_option("-t", "--filename-tree", dest="filename_tree", type="string",
                      help="filename with tree(s)." )
    parser.add_option("--skip-header", dest="add_header", action="store_false",
                      help="do not add header to flat format."  )
    parser.add_option("--write-header", dest="write_header", action="store_true",
                      help="write header and exit."  )
    parser.add_option("--debug", dest="debug", action="store_true",
                      help="debug mode")
    parser.add_option("--display-tree", dest="display_tree", action="store_true",
                      help="display the tree")
    
    parser.add_option("-m", "--method", dest="methods", type="choice", action="append",
                      choices=("contrasts", "spearman", "pearson", "compute" ),
                      help="methods to perform on contrasts." )
    
    parser.set_defaults(
        columns = "all",
        filename_tree = None,
        add_header = True,
        write_header = False,
        debug = False,
        methods = [],
        value_format = "%6.4f",
        pvalue_format = "%e",
        display_tree = False,
        )

    (options, args) = E.Start( parser, quiet = True )

    if options.columns not in ( "all", "all-but-first"):
        options.columns = map(lambda x: int(x) -1 , options.columns.split(","))

    phylip = WrapperPhylip.Phylip()
    
    if options.debug:
        phylip.setLogLevel( options.loglevel )

    phylip.setProgram( "contrast" )

    ##########################################################
    ##########################################################
    ##########################################################        
    # retrieve data and give to phylip
    data = []    
    headers = []
    first = True
    for line in sys.stdin:
        if line[0] == "#": continue
        d = line[:-1].strip().split("\t") 
        if first:
            first = False
            headers = d[1:]
            continue
        data.append(d)
        
    phylip.setData( data )
    ncolumns = len(headers)
    nrows = len(data)
    
    ##########################################################
    ##########################################################
    ##########################################################        
    # read trees
    nexus = None
    if options.filename_tree:
        nexus = TreeTools.Newick2Nexus( open(options.filename_tree, "r") )

    if not nexus:
        raise ValueError("please provide trees with branchlenghts")
    
    ##########################################################
    ##########################################################
    ##########################################################
    # set up phylip
    phylip_options = []
    ## print out contrasts
    phylip_options.append("C")        
    phylip_options.append("Y")        
    phylip.setOptions( phylip_options )
    
    ##########################################################
    ##########################################################
    ##########################################################
    ## main loop
    ##########################################################    
    for tree in nexus.trees:

        if options.display_tree:
            tree.display()

        ## compute this before giving the tree to the phylip module,
        ## as it remaps taxon names.
        map_node2data = {}
        for x in range(nrows):
            taxon = data[x][0]
            map_node2data[tree.search_taxon( taxon )] = x
        
        phylip.setTree( tree )
        
        result = phylip.run()

        for method in options.methods:

            if method in ("pearson", "spearman"):

                options.stdout.write( "header1\theader2\tr\tp\tcode\n" )
                
                n = len(result.mContrasts)
                columns = []
                for c in range( ncolumns ):
                    columns.append( map( lambda x: x[c], result.mContrasts ) )

                for x in range( 0, ncolumns - 1):
                    for y in range( x+1, ncolumns) :
                        
                        # phylip value
                        phy_r = result.mCorrelations[x][y]

                        import rpy
                        from rpy import r as R
                        
                        ## Various ways to calculate r. It is not possible to use
                        ## cor.test or lsfit directly, as you have to perform a
                        ## regression through the origin.
                        
                        ## uncomment to check pearson r against phylip's value
                        ## r = calculateCorrelationCoefficient( columns[x], columns[y] )

                        ## for significance, use linear regression models in R
                        rpy.set_default_mode(rpy.NO_CONVERSION)
                        linear_model = R.lm(R("y ~ x - 1"), data = R.data_frame(x=columns[x], y=columns[y]))
                        rpy.set_default_mode(rpy.BASIC_CONVERSION)

                        ss = R.summary(linear_model)

                        ## extract the p-value
                        p = ss['coefficients'][-1][-1]

                        if p < 0.001:
                            code = "***"
                        elif p < 0.01:
                            code = "**"
                        elif p < 0.05:
                            code = "*"
                        else:
                            code = ""

                        options.stdout.write( "\t".join( (headers[x], headers[y],
                                                          options.value_format % phy_r,
                                                          options.pvalue_format % p,
                                                          code)) + "\n" )

            elif method == "contrasts":
                
                options.stdout.write( "\t".join(headers) + "\n" )
                for d in result.mContrasts:
                    options.stdout.write( "\t".join(map( lambda x: options.value_format % x, d )) + "\n ")

            elif method == "compute":

                ## make room for all internal nodes and one dummy node
                ## for unrooted trees.
                max_index = TreeTools.GetMaxIndex( tree ) + 2
                variances = [None] * max_index
                values = [ [None] * nrows for x in range(max_index) ]
                contrasts = []
                for x in range(max_index):
                    contrasts.append( [None] * ncolumns )
                branchlengths = [None] * max_index
                
                def update_data( node_id, bl, c1, c2, ):

                    b1, b2 = branchlengths[c1], branchlengths[c2]
                    rb1 = 1.0/b1
                    rb2 = 1.0/b2
                    # compute variance
                    variance = math.sqrt(b1+b2)

                    # extend branch length of this node to create correct variance for parent
                    branchlengths[node_id] = bl + (b1 * b2) / (b1 + b2)
                    variances[node_id] = variance

                    for c in range(ncolumns):
                        v1, v2 = values[c1][c], values[c2][c]
                        # save ancestral value as weighted mean
                        values[node_id][c] = (( rb1*v1 + rb2*v2) ) / (rb1+rb2)
                        # compute normalized contrast
                        contrasts[node_id][c] = (v1-v2) / variance
                
                def update_contrasts( node_id ):
                    """update contrasts for a node."""
                    node = tree.node(node_id)
                    if node.succ:
                        if len(node.succ) == 2:
                            c1, c2 = node.succ
                            update_data( node_id, node.data.branchlength, c1, c2 )
                        else:
                            assert( node_id == tree.root )
                            assert( len(node.succ) == 3 )
                            update_data( node_id, node.data.branchlength, node.succ[0], node.succ[1] )
                            update_data( max_index - 1, node.data.branchlength, node_id, node.succ[2] )
                    else:
                        for c in range(ncolumns):
                            values[node_id][c] = float(data[map_node2data[node_id]][c+1])
                            
                        branchlengths[node_id] = node.data.branchlength
                    
                tree.dfs( tree.root, post_function = update_contrasts )

                options.stdout.write( "node_id\tvariance\t%s\n" % "\t".join(headers) )
                for node_id in range(max_index):
                    if variances[node_id] == None: continue
                    options.stdout.write( "%s\t%s\t%s\n" % ( node_id,
                                                             options.value_format % variances[node_id],                                                             
                                                             "\t".join( map( lambda x: options.value_format % x, contrasts[node_id])),
                                                             ))
                                                                                 
                                                                                      
                
    E.Stop()
    
    













################################################################################
#   Gene prediction pipeline 
#
#   $Id: tree2patterns.py 2781 2009-09-10 11:33:14Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
tree2patterns.py - convert trees to taxon patterns
==================================================

:Author: Andreas Heger
:Release: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a collections of trees from stdin and - based
on a reference tree - outputs for each tree a pattern indicating
if a certain taxon has been found or net.

``1`` indicates that a pattern has been found, ``0`` otherwise.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 

import os
import sys
import string
import re
import getopt
import time
import sets
import optparse
import math
import tempfile


from Bio.Nexus import Nexus
from Bio.Nexus.Nodes import Node

import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: tree2patterns.py 2781 2009-09-10 11:33:14Z andreas $",
                                    usage = globals()["__doc__"] )

    parser.add_option("-t", "--reference-tree", dest="reference_tree", type="string",
                      help="reference tree to read."  )

    parser.add_option("-s", "--sort-order", dest="sort_order", type="string",
                      help="output order of OTU."  )

    parser.set_defaults(
        reference_tree = None,
        sort_order = [],
        )

    (options, args) = E.Start( parser )

    if not options.sort_order:
        for nx in reference_tree.get_terminals():
            options.sort_order.append( reference_tree.node(nx).get_data().taxon )
    else:
        options.sort_order = options.sort_order.split(",")

    if not options.reference_tree:
        raise "no reference tree defined."

    nexus = TreeTools.Newick2Nexus( options.reference_tree )
    reference_tree = nexus.trees[0]
    
    if options.loglevel >= 3:
        print "# reference tree:"
        print reference_tree.display()

    patterns = TreeTools.calculatePatternsFromTree( tree, options.sort_order )

    for p in patterns:
        print p

    E.Stop()






################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
rnaseq_bams_vs_bams.py - compare multiple bam files against each other
=======================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Compare RNASeq reads in multiple BAM files against each other.

.. note::
    BAM files need to be sorted by read name. samtools sort
    does NOT work as it uses a custom comparison function
    (strnum_cmp) that is incompatible with the standard 
    lexicographical order in python. See the example below
    on how to get sorted files.

This script is for validation purposes. It might take a while
for large BAM files.

Usage
-----

If you have two sorted :term:`sam` or :term:`bam` formatted
files, type::

   python rnaseq_bams_vs_bams.py a.bam b.bam > out

If they are not sorted, you can use samtools sort to do an
inplace sort::

   python rnaseq_bams_vs_bams.py <( samtools view -h a.bam | hsort 0 -k1,1) 
                  <( samtools view -h b.bam | hsort 0 -k1,1) 

The ``-h`` option outputs the header, and the hsort command sorts without 
disturbing the header. 

Type::

   python rnaseq_bams_vs_bams.py --help

for command line help.

Documentation
-------------

For read counts to be correct the NH flag to be set correctly.

Code
----

'''

import os
import sys
import re
import optparse
import collections
import itertools
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import pysam
import CGAT.GTF as GTF

class multiway_groupby(object):
    # [k for k, g in groupby('AAAABBBCCDAABBB')] --> A B C D A B
    # [list(g) for k, g in groupby('AAAABBBCCD')] --> AAAA BBB CC D
    def __init__(self, iterables, key=None):
        # set up iterators
        self.it = [ itertools.groupby(iterable, key) for iterable in iterables ]

        # todo: catch StopIterator for empty iterables
        self.current = [ next(self.it[x]) for x in xrange(len(iterables)) ]

        # decide on target key
        self.targetkey = min( [x[0] for x in self.current if x[0] != None ] )

    def __iter__(self):
        return self

    def next(self):

        # check if all iterators exhausted
        if not any( self.it ): raise StopIteration

        # yield all that correspond to target key and update
        result = []
        targetkey = self.targetkey
        updated = 0

        for x, y in enumerate( self.current ):
            key, val = y
            if key and key == targetkey: 
                # save result (instantiate to prevent overwriting)
                # in next method
                result.append( list(val) )
                # advance
                try:
                    self.current[x] = next(self.it[x])
                except StopIteration:
                    self.it[x] = None
                    self.current[x] = (None,None)
                updated += 1
            else: 
                # return empty result
                result.append( [] )

        assert updated > 0, "no updates - infinite loop"

        # decide which is target key
        try: 
            self.targetkey = min( [x[0] for x in self.current if x[0] != None ] )
        except ValueError:
            # if all are None, sequence is empty
            self.targetkey = None
        
        return targetkey, result

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-e", "--headers", dest="headers", type="string",
                       help = "',' separated list of labels used as headers. "
                              " Should correspond in order to command line arguments [%default]" )

    parser.set_defaults(
        headers = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    read_length = 50
    max_distance = read_length

    if len(args) < 2:
        raise ValueError( "please specify at least two BAM files" )

    infiles = []
    for arg in args: infiles.append( pysam.Samfile( arg, 'r' ) )

    if options.headers:
        headers = options.headers.split(",")
        if len(headers) != len(args):
            raise ValueError("number of headers and files differrent")
    else:
        headers = [ "file%i" % x for x in range( 1, len(infiles) + 1) ]

    options.stdout.write( "read\tnlocations\tnmatched\t%s\t%s\n" % \
                              ("\t".join( ["%s_nh" % x for x in headers] ),
                               "\t".join( ["%s_loc" % x for x in headers] ) ) )

    ninput, noutput = 0, 0
    for readname, result in multiway_groupby( infiles, key = lambda x: x.qname ):
        ninput += 1
        nmatches = 0
        locations = set()

        # build location directory
        for r in result: 
            for rr in r: 
                if rr.is_unmapped: continue
                locations.add( (rr.tid, rr.pos) )

        # permit a certain fuzzyness in locations
        # as some mappers clip and others don't.
        locations = list(locations)
        locations.sort()
        pos2loc = {}
        last_tid, last_pos, nlocations = None, 0, -1
        for tid, pos in locations:
            if tid != last_tid or pos - last_pos > max_distance:
                nlocations += 1
            pos2loc[(tid,pos)] = nlocations
            last_tid, last_pos = tid, pos
        nlocations += 1

        # code locations
        codes, nh = [], []
        for r in result:
            c = []
            for rr in r:
                if rr.is_unmapped: continue
                c.append( pos2loc[ (rr.tid, rr.pos) ] )
            codes.append( ",".join( map(str, sorted(c)) ) )
            nh.append( len(c) )
            if len(c): nmatches += 1

        noutput += 1
        options.stdout.write( "%s\t%i\t%i\t%s\t%s\n" % (readname, 
                                                        nlocations,
                                                        nmatches, 
                                                        "\t".join([ "%i" % x for x in nh ]),
                                                        "\t".join( codes ) ) )

    E.info( "ninput=%i, noutput=%i" % (ninput,noutput) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
clean.py - clean up output files from aborted runs
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script checks one or more output files have they
have completed successfully. It will remove output files
for those jobs that are incomplete.

The script checks for the "job finished" tag at the
end of the file.

Usage
-----

Example::

   python clean.py --help

Type::

   python clean.py --help

for command line help.

Documentation
-------------

Code
----
'''

import os
import sys
import re
import string
import optparse
import glob
import subprocess
import os.path
import CGAT.Experiment as E

def getLastLine( filename, read_size = 1024 ):
  """return last line of a file.
  """
  f = open(filename, 'rU')    # U is to open it with Universal newline support
  offset = read_size
  f.seek(0, 2)
  file_size = f.tell()
  if file_size == 0: return ""
  while 1:
    if file_size < offset:
      offset = file_size
    f.seek(-1*offset, 2)
    read_str = f.read(offset)
    # Remove newline at the end
    if read_str[offset - 1] == '\n':
      read_str = read_str[:-1]
    lines = read_str.split('\n')
    if len(lines) >= 2:
        return lines[-1]
    if offset == file_size:   # reached the beginning
      return read_str
    offset += read_size
  f.close()

def checkPythonRuns( filename ):
    """returns true if a python run is complete."""
    last_line = getLastLine( filename )
    return re.match( "# job finished", last_line )
    
def isNewer( a, b ):
    """return true if file a is newer than file b."""

    ## get times of most recent access
    at = os.stat( a )[7]
    bt = os.stat( b )[7]

    return at > bt

if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: clean.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"] )

    parser.add_option( "-g", "--glob", dest="glob_pattern", type="string" ,
                       help="glob pattern to use for collecting files [%default].")

    parser.add_option( "-n", "--dry-run", dest="dry_run", action="store_true",
                       help="only print out actions, do not execute them [%default]." )

    parser.add_option( "-f", "--file-pattern", dest="file_pattern", type="string",
                       help="only check files matching this pattern [%default]." )

    parser.set_defaults( glob_pattern = "data.dir",
                         file_pattern = ".out",
                         check_completeness = "python",
                         skip_dirs = [],
                         dry_run = False,
                         )

    (options, args) = E.Start( parser, 
                                        add_pipe_options = True )

    if args:
        starts = args
    elif options.glob_pattern:
        starts = glob.glob( options.glob_pattern )
    else:
        starts = "."

    ndirs, nfiles, ndeleted = 0, 0, 0

    if options.check_completeness == "python":
        isComplete = checkPythonRuns

    rx = re.compile( options.file_pattern )

    for start in starts:
        for root, dirs, files in os.walk(start):
            
            ndirs += 1
            ## exclude directories
            for dir in options.skip_dirs:
                if dir in dirs:
                    dirs.remove(dir) 

            for filename in files:
                p = os.path.join(root,filename ) 
                if rx.search( filename ) and not isComplete( p ):
                    if options.dry_run:
                        options.stdlog.write("# removing file %s\n" % p )
                    else:
                        os.remove( p )
                    ndeleted += 1

    if options.loglevel >= 1:
        options.stdlog.write("# ndirs=%i, nfiles=%i, ndeleted=%i\n" %\
                                 (ndirs, nfiles, ndeleted) )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Tophat.py - working with tophat/cufflinks output files
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''

import os, sys, re, collections, itertools

import IOTools

class CuffCompareValues:
    def __init__( self, vals ):

        assert len(vals) == 4
        try: self.sn = float( vals[0] )
        except ValueError: self.sn = None

        try: self.sp = float( vals[1] )
        except ValueError: self.sp = None

        try: self.fsn = float( vals[2] )
        except ValueError: self.fsn = None

        try: self.fsp = float( vals[3] )
        except ValueError: self.fsp = None
    
    def __str__(self):
        return "\t".join( map(IOTools.val2str,
                              ( self.sn,
                                self.sp,
                                self.fsn,
                                self.fsp ) ) )
    @classmethod
    def getHeaders( cls ):
        return ("sn", "sp", "fsn", "fsp" )

class CuffCompareResult:

    def __init__( self ):
        ( self.baselevel,
          self.exonlevel,
          self.intronlevel,
          self.intronchainlevel,
          self.transcriptlevel,
          self.locuslevel,
          self.missedexons_counts,
          self.missedexons_total,
          self.novelexons_counts,
          self.novelexons_total,
          self.missedintrons_counts,
          self.missedintrons_total,
          self.novelintrons_counts,
          self.novelintrons_total,
          self.missedloci_counts,
          self.missedloci_total,
          self.novelloci_counts,
          self.novelloci_total,
          self.matchingintronchains_count,
          self.matchingloci_count,
          self.query,
          self.query_loci,
          self.query_multi_exon,
          self.reference,
          self.reference_loci,
          self.reference_multi_exon,
          self.loci_multi_exon,
          self.loci_transcripts,
          ) = [None] * 28

    def fromLines( self, lines ):
        '''parse from cuffcompare output.'''
        self.is_empty = True

        for line in lines:
            if line.startswith("#"):
                d = line[1:-1].strip()
                if d.startswith("Query"):
                    self.query, self.query_loci, self.query_multi_exon = map(int, re.match( \
                        "Query mRNAs :\s+(\d+)\s+in\s+(\d+)\s+loci\s+\((\d+)", d ).groups() )
                elif d.startswith("Reference"):
                    self.reference, self.reference_loci, self.reference_multi_exon = map( int, re.match( \
                        "Reference mRNAs :\s+(\d+)\s+in\s+(\d+)\s+loci\s+\((\d+)", d ).groups() )
                elif d.startswith( "("):
                    self.loci_multi_exon, self.loci_transcripts = re.match( \
                        "\((\d+) multi-transcript loci, ~(\S+)", d ).groups()
                    self.loci_multi_exon = int( self.loci_multi_exon )
                    self.loci_transcripts = float( self.loci_transcripts )
                continue

            line = line[:-1].strip()
            if not line: continue

            try:
                tag, data = line.split(":")
            except ValueError:
                raise ValueError("parsing error in line %s" % line )



            tag = re.sub( "\s", "", tag ).lower()
            
            if tag.startswith( "novel" ) or tag.startswith("missed"):

                counts, total = map(int, re.match( "\s+(\d+)/(\d+)", data).groups() )
                setattr( self, "%s_counts" % tag, counts )
                setattr( self, "%s_total" % tag, total )
                

            # NICK FIX: This output changed from the previous version of cufflinks. not sure if it means the same
            # but nevertheless the new output will be put into sphinxreport
            elif tag.startswith("matching"):
                counts = int(data)
                setattr( self, "%s_count" % tag, counts )

            elif tag.startswith( "total"): 
                # stop at total union across all super-loci
                break
            else:
                values = data.strip().split()
                setattr( self, tag, CuffCompareValues( values ) )
                self.is_empty = False
            
    def __str__(self):
        return "\t".join( map(str, (
                    self.baselevel,
                    self.exonlevel,
                    self.intronlevel,
                    self.intronchainlevel,
                    self.transcriptlevel,
                    self.locuslevel,
                    self.missedexons_counts,
                    self.missedexons_total,
                    self.novelexons_counts,
                    self.novelexons_total,
                    self.missedintrons_counts,
                    self.missedintrons_total,
                    self.novelintrons_counts,
                    self.novelintrons_total,
                    self.missedloci_counts,
                    self.missedloci_total,
                    self.novelloci_counts,
                    self.novelloci_total,
                    self.matchingintronchains_count,
                    self.matchingloci_count,
                    self.query,
                    self.query_loci,
                    self.query_multi_exon,
                    self.reference,
                    self.reference_loci,
                    self.reference_multi_exon,
                    self.loci_multi_exon,
                    self.loci_transcripts
                    ) ) )

    @classmethod
    def getHeaders( cls ):
        m = ( "baselevel", "exonlevel", "intronlevel", "intronchainlevel",
              "transcriptlevel", "locuslevel" )
        a = ( "missed", "novel")
        b = ( "exons", "introns", "loci" )
        c = ( "counts", "total" )
        return [ "%s_%s" % (x,y) for x,y in itertools.product( m, CuffCompareValues.getHeaders() ) ] +\
            [ "%s%s_%s" % (x,y,z) for x,y,z in itertools.product( a, b, c) ] +\
            ["matchingintronchains_count", "matchingloci_count"] +\
            [ "query", "query_loci", "query_multi_exon",
              "reference", "reference_loci", "reference_multi_exon",
              "loci", "loci_multi_exon" ]
    
def parseTranscriptComparison( infile ):
    '''read cufflinks 1.0.3 output in infile stream.

    returns a two-level dictionary mapping with levels track and contig.
    '''

    
    firstline = infile.readline()
    result = collections.defaultdict( dict )
    tracks = []

    def __blocker( infile ):
        # 'Summary for dataset'
        # comes after 'Genomic sequence'.

        blocks = {}
        contig, block = None, []
        # whether to yield
        y = False
        for line in infile:

            if line.startswith("#> Genomic sequence"):
                if block: blocks[contig] = block
                if y:
                    yield dataset, blocks
                    blocks = {}
                    y = False

                contig = re.match("#> Genomic sequence: (\S+)", line).groups()[0]
                block = []
                continue
            elif line.startswith( "#= Summary for dataset:" ):
                if block: blocks[contig] = block
                if y:
                    yield dataset, blocks
                    blocks = {}
                    y = False

                dataset = re.match("#= Summary for dataset: (\S+)", line).groups()[0]
                contig = "all"
                block = []
                
                y = True
                continue

            block.append( line )

        blocks[contig] = block
        yield dataset, blocks

    for track, blocks in __blocker( infile ):
        print track
        tracks.append( track )
        for contig, block in blocks.iteritems():
            r = CuffCompareResult()
            
            r.fromLines( block )
            result[track][contig] = r

    return tracks, result

Locus = collections.namedtuple( "Locus", "locus_id contig strand start end transcript_ids transcripts" )
Tracking = collections.namedtuple( "Tracking", "transfrag_id locus_id ref_gene_id ref_transcript_id code transcripts" )
TranscriptInfo = collections.namedtuple( "Transfrag", "gene_id transcript_id fmi fpkm conf_lo conf_hi cov len" )

def iterate_tracking( infile ):
    '''parse .tracking output file from cuffcompare

    returns iterator with list of loci.
    '''
    for line in infile:
        data = line[:-1].split("\t")
        try:
            transfrag_id, locus_id, transcript_id, code = [x.strip() for x in data[:4]]
        except ValueError:
            raise ValueError("parsing error: expected transfrag_id, locus_id, transcript_id, code, got %s" % str(data))

        if transcript_id == "-":
            ref_gene_id, ref_transcript_id = "", ""
        else:
            ref_gene_id, ref_transcript_id = transcript_id.split("|")

        transcripts = []
        for c in data[4:]:
            if c == "-": 
                transcripts.append( None )
            else: 
                # there can be multiple transcripts
                for cc in c.split( "," ):
                    try:
                        (gene_id, transcript_id, fmi, fpkm, conf_lo, conf_hi, cov, length )= cc.split("|")
                    except ValueError:
                        raise ValueError( "parsing error for field '%s'" % cc )
                    (fpkm, conf_lo, conf_hi, cov) = map(float, (fpkm, conf_lo, conf_hi, cov))
                    if length == "-": length = 0
                    (fmi, length) = map( int, (fmi, length) )

                    transcripts.append( TranscriptInfo._make( 
                            ( gene_id, transcript_id, fmi, fpkm, conf_lo, conf_hi, cov, length )) )
                
        yield Tracking._make( (transfrag_id, locus_id, ref_gene_id, ref_transcript_id, code, transcripts ) )

                                    
def iterate_locus( infile ):
    '''parse .loci output file from cuffcompare

    returns iterator with list of loci.
    '''
                                    
    for line in infile:
        data = line[:-1].split("\t")
        locus_id, pos = data[:2]
                                    
        contig, strand, start, end = re.match( "(\S+)\[(\S*)\](\d+)-(\d+)", pos ).groups()
        start, end = map(int, (start, end ) )

        # some [] contains the 0 byte, convert to empty field
        if strand not in "+-": strand = ""
        transcripts = []
        for c in data[2:]:
            if c == "-": 
                transcripts.append( () )
            else: 
                transcripts.append( c.split(",") )

        yield Locus._make( (locus_id, contig, strand, start, end, transcripts[1], transcripts[1:] ) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
rnaseq_bam_vs_bed.py - count context that reads map to
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes as input a :term:`BAM` file from an RNASeq experiment 
and a :term:`bed` formatted file.

It counts the number of alignments overlapping between the :term:`bam`
file and the :term:`bed` file. Annotations in the :term:`bed` file can
be overlapping - they are counted independently per name.

This scripts requires bedtools to be installed.

Usage
-----

Example::

   python cgat_script_template.py in.bam in.bed.gz

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------


Code
----

'''

import os
import sys
import re
import optparse
import time
import subprocess
import tempfile
import collections
import itertools

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import pysam
import CGAT.Bed as Bed

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "-m", "--min-overlap", dest="min_overlap", type="float",
                       help = "minimum overlap [%default]" )

    parser.add_option( "-k", "--keep-temp", dest="keep_temp", action="store_true",
                       help = "do not delete temporary files [%default]" )

    parser.set_defaults(
        min_overlap = 0.5,
        keep_temp = False,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) != 2:
        raise ValueError( "please supply a bam and a bed file or two bed-files." )
    
    bamfile, bedfile = args
    
    E.info( "intersecting the two files" )

    tmpfile = tempfile.NamedTemporaryFile( delete=False )
    tmpfile.close()
    tmpfilename = tmpfile.name

    min_overlap = options.min_overlap

    options.stdout.write( "category\talignments\n" )

    # get number of columns of reference bed file
    for bed in Bed.iterator(IOTools.openFile( bedfile )):
        ncolumns_bed = bed.columns
        break
    E.info( "assuming %s is bed%i format" % (bedfile, ncolumns_bed))

    if ncolumns_bed < 4:
        raise ValueError("please supply a name attribute in the bed file")

    # get information about 
    if bamfile.endswith(".bam"):
        format = "-abam"
        samfile = pysam.Samfile( bamfile, "rb" )
        total = samfile.mapped
        # latest bedtools uses bed12 format when bam is input
        ncolumns_bam = 12
        # count per read
        sort_key = lambda x: x.name
    else: 
        format = "-a"
        total = IOTools.getNumLines( bamfile )
        # get bed format
        ncolumns_bam = 0
        for bed in Bed.iterator(IOTools.openFile( bamfile )):
            ncolumns_bam = bed.columns
            break

        if ncolumns_bam > 0:
            E.info( "assuming %s is bed%i fomat" % (bamfile, ncolumns_bam ))
            if ncolumns_bam == 3:
                # count per interval
                sort_key = lambda x: (x.contig, x.start, x.end)
            else:
                # count per interval category
                sort_key = lambda x: x.name

    # use fields for bam/bed file (regions to count with)
    data_fields = [ "contig", "start", "end", "name",
                    "score", "strand", "thickstart", "thickend", "rgb",
                    "blockcount", "blockstarts", "blockends" ][:ncolumns_bam]

    # add fields for second bed (regions to count in)
    data_fields.extend( [ "contig2", "start2", "end2", "name2",
                          "score2", "strand2", "thickstart2", "thickend2", "rgb2",
                          "blockcount2", "blockstarts2", "blockends2" ][:ncolumns_bed] )

    # add bases overlap
    data_fields.append( "bases_overlap"  )

    data = collections.namedtuple( "data", data_fields )

    options.stdout.write( "total\t%i\n" % total )

    if total == 0:
        E.warn( "no data in %s" % bamfile )
        return

    statement = """intersectBed %(format)s %(bamfile)s -b %(bedfile)s -bed -wo -f %(min_overlap)f > %(tmpfilename)s""" % locals()

    E.info( "running %s" % statement )
    retcode = E.run( statement )

    if retcode != 0:
        raise ValueError( "error while executing statement %s" % statement )

    infile = open( tmpfilename, "r")
    counts_per_alignment = collections.defaultdict(int)

    E.info( "counting" )

    take_columns = len(data._fields)

    def iter( infile ):
        for line in infile:
            if not line.strip(): continue
            yield data._make( line[:-1].split()[:take_columns] )

    for read, overlaps in itertools.groupby( iter(infile), key = sort_key ):
        annotations = [x.name2 for x in overlaps ]
        for anno in annotations:
            counts_per_alignment[anno] += 1
    infile.close()

    for key, counts  in counts_per_alignment.iteritems():
        options.stdout.write( "%s\t%i\n" % (key, counts) )

    if not options.keep_temp:
        os.unlink( tmpfilename )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
combine_histograms.py - combine several histograms
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

OPTIONS:
-n, --normalize         normalize each histogram
-m, --missing=          set missing values to #
-t, --titles            use supplied titles
-h, --headers=          set header to #, "auto" = filenames
-f, --format=           format to use when combining histograms
--format-value=         format for numbers
--format-bin=           format for bin values

-> relative frequencies
-> cumulative counts and frequencies in both directions

Usage
-----

Example::

   python combine_histograms.py --help

Type::

   python combine_histograms.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time

param_long_options = ["missing=", "headers=", "titles", "normalize", 
                      "format=", "format-bin=", "format-value=", "sort=", "help",
                      "version"]
param_short_options = "v:ht:m:h:s:f:"

param_headers = None
param_normalize = 0

param_missing_value = 0

param_titles = True

param_format = None
param_format_bin = None
param_format_value = None

param_sort = None

import CGAT.Experiment as E
import CGAT.Histogram as Histogram

##---------------------------------------------------------------------------------------------------------        
if __name__ == '__main__':

    try:
        optlist, args = getopt.getopt(sys.argv[1:],
                                      param_short_options,
                                      param_long_options)
                                      

    except getopt.error, msg:
        print globals()["__doc__"], msg
        sys.exit(1)

    for o,a in optlist:
        if o in ( "--help",):
            print globals()["__doc__"]
            sys.exit(0)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ("-h", "--headers"):
            param_headers = string.split( a, "," )
        elif o in ("-n", "--normalize"):
            param_normalize = 1
        elif o in ("-m", "--missing"):
            param_missing_value = a
        elif o == "--no-titles":
            param_titles = False
        elif o == "--no-titles":
            param_titles = False
        elif o in ("-f", "--format"):
            param_format = a
        elif o == "--format-value":
            param_format_value = a
        elif o == "--format-bin":
            param_format_bin = a
        elif o in ("-s", "--sort"):
            if a in ("numerical", "alphabetic"):
                param_sort = a
            else:
                param_sort = string.split( a, "," )

    if len(args) < 1:
        print globals()["__doc__"], "please specify at one histogram."
        sys.exit(1)

    param_filenames = args

    print E.GetHeader()
    print E.GetParams()    

    histograms = []

    ## first
    headers = ['bin',]
    if param_headers and headers != "auto":
        headers = [param_headers[0],]
        del param_headers[0]
        
    for x in range(len(param_filenames)):
        
        filename = param_filenames[x]
        if not os.path.exists(filename):
            print "# skipped because file not present: %s" % filename
            continue

        file = open(filename, "r")

        lines = filter( lambda x: x[0] <> "#", file)
        
        if len(lines) == 0:
            continue

        if param_titles:
            h = lines[0][:-1].split("\t")[1:]
            del lines[0]
                
        if param_headers == "auto":
            headers.append( os.path.basename( filename ) )
        elif param_headers:
            headers.append( param_headers[x] )
        elif param_titles:
            headers += h

        data = map( lambda x: map(string.atof, string.split(x[:-1], "\t")), lines)

        ## add empty data point for empty histograms
        if len(data) == 0: data = [ (0,0) ] 
        
        histograms.append( data )

    ## sort the whole thing:
    if param_sort:
        sort_order = []

        if param_sort == "numerical":
            t = zip( map(int, headers[1:]), range( 1, len(headers) + 1) )
            t.sort()

            for tt in t:
                sort_order.append( headers[tt[1]] )

        elif param_sort == "alphabetical":
            t = zip( headers[1:], range( 1, len(headers) + 1) )
            t.sort()

            for tt in t:
                sort_order.append( headers[tt[1]] )
        else:
            sort_order = param_sort

        ## map header to old position
        map_header2pos = {}
        for x in range(1, len(headers)):
            map_header2pos[headers[x]] = x

        order = []
        for x in sort_order:
            if x in map_header2pos:                
                order.append( map_header2pos[x] )

        new_headers = [headers[0]]
        new_histograms = []

        for x in order:
            new_headers.append( headers[x] )
            new_histograms.append( histograms[x-1] )

        histograms = new_histograms
        headers = new_headers
        
    combined_histogram = Histogram.Combine( histograms, param_missing_value )

    if headers:
        print "\t".join(headers)

    if param_normalize:
        combined_histogram = Histogram.Normalize( combined_histogram )

    Histogram.Print( combined_histogram,
                     format_bin=param_format_bin,
                     format_value=param_format_value,
                     )

    print E.GetFooter()





################################################################################
#   Gene prediction pipeline 
#
#   $Id: tree_collapse_species.py 2782 2009-09-10 11:40:29Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
tree_collapse_species.py - collapse single species nodes 
========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script reads a collection of trees from stdin and 
collapses nodes that have children of the same species.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

""" 

import os
import sys
import string
import re
import getopt
import tempfile
import time
import popen2

from Bio.Nexus import Nexus

USAGE="""python %s [OPTIONS] < tree.in > tree.out

Version: $Id: tree_collapse_species.py 2782 2009-09-10 11:40:29Z andreas $



Options:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
-p, --pattern-species=          regex pattern to extract species from identifier
""" % sys.argv[0]

import CGAT.Experiment as E

param_loglevel = 1

param_long_options=["verbose=", "help",
                    "pattern-species=",
                    "version"]

param_short_options="v:hp:"

param_pattern_species = "^([^@:]+)[@:]"


def PruneTree( tree, id ):

    if id not in tree.get_terminals():
        raise "Not a terminal taxon: %i" % id
    
    prev=tree.unlink(id)
    tree.kill(id)
    if not prev==tree.root and len(tree.node(prev).succ)==1:
        
        succ=tree.node(prev).get_succ()[0]
        new_bl=tree.node(prev).data.branchlength+tree.node(succ).data.branchlength
        tree.collapse(prev)
        tree.node(succ).data.branchlength=new_bl
        
    return prev

if __name__ == "__main__":

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print USAGE, msg
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "-h", "--help" ):
            print USAGE
            sys.exit(0)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ("-p", "--pattern-species"):
            param_pattern_species = a

    print E.GetHeader()
    print E.GetParams()

    lines = ["#NEXUS\nBegin trees;\ntree tree = "] + sys.stdin.readlines() + ["End;"]
    nexus = Nexus.Nexus( string.join(lines, "") )

    if len(nexus.trees) != 1:
        raise "no tree found in file."
    
    tree = nexus.trees[0]
    
    if param_loglevel >= 2:
        tree.display()

    rx = re.compile( param_pattern_species )
    changed = True
    
    while changed:
        changed = False
        leaves = tree.get_terminals()
        for x in range(0, len(leaves) -1 ):
            nx = leaves[x]
            t1 = tree.node(nx).get_data().taxon
            s1 = rx.search( t1 ).groups()[0]
            p1 = tree.node(nx).get_prev()            
            for y in range( x + 1, len(leaves)):
                ny = leaves[y]
                t2 = tree.node(ny).get_data().taxon                
                s2 = rx.search( t2 ).groups()[0]
                p2 = tree.node(ny).get_prev()
                if s1 == s2 and tree.is_monophyletic( (nx, ny) ) != -1:
                    print "collapsing nodes", t1, t2, nx, ny, p1, p2                    
                    PruneTree( tree, nx )
                    if param_loglevel >= 2:
                        tree.display()
                    changed = True
        
    print E.GetFooter()



"""
Bioprospector.py - Parser for Bioprospector output
==================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python
"""

import string, sys, re, types, collections

import weblogolib
import corebio.seq

Match = collections.namedtuple( "Match",
                                  '''id, site, strand, start, end,
                                  width1, width2, sequence''')

Motif = collections.namedtuple( "Motif",
                                '''id, pattern, width1, width2,
                                min_gap, max_gap,
                                score, sites, matches''')

                                        
def parse( infile ):
    '''parse Bioprospector output.'''

    motifs = []
    
    def iterate_blocks(infile):
        lines = []
        keep = False
        for line in infile:
            if line.startswith( "Motif #"):
                if lines: yield lines
                keep = True
                lines = []
                
            if line.startswith("*****"):
                continue

            if keep:
                lines.append( line )
        if lines: yield(lines)
        
    for lines in iterate_blocks(infile):
        motif_id, pattern = re.match( "Motif \#(\d+): \((.+)\)", lines[0]).groups()
        widths, gaps, score, sites = re.match("Width \((.*)\); Gap \[(.*)\]; MotifScore (.*); Sites (.*)", lines[1]).groups()
        width1, width2 = map(int, widths.split(","))
        min_gap, max_gap = map(int, gaps.split(","))

        sites = int(sites)
        start = end = 3
        while not lines[end].startswith(">"): end += 1

        block = lines[start:end]

        def convert( coords, width ):
            if coords[0] == "f":
                strand,start = "+", int(coords[1])-1
            else:
                strand,start = "-", int(coords[1]) - width
            return strand, start
        
        sequences = []
        for x in range( end, len(lines), 2):
            if lines[x].strip() == "": break
            assert lines[x].startswith(">"), lines[x]
            id, lseq, site_id, coords = re.match(">(\S+).*len\s(\d+).*site \#(\d+)\s*(.*)", lines[x][:-1]).groups()
            lseq, site_id = map(int, (lseq, site_id) )
            coords = re.split("\s", coords )
            if len(coords) == 2:
                strand1,start1 = convert(coords[:2], width1)
                strand = strand1
                start, end = start, start + width1
            elif len(coords) == 4:
                strand1,start1 = convert(coords[:2], width1)
                strand2,start2 = convert(coords[2:], width2)
                start, end = min(start1, start2), max(start1 + width1,start2 + width2)
                strand = strand1 + strand2
                
            seq = lines[x+1][:-1]
            sequences.append( Match._make( (id, site_id,
                                            strand,
                                            start, end,
                                            width1, width2,
                                            seq ) ) )
            
        motif = Motif._make( (motif_id, pattern,
                              width1, width2,
                              min_gap, max_gap,
                              score,
                              sites, sequences ) )
        motifs.append( motif )

    return motifs
                             
def build_logo( sequences, outfilename ):

    seqs = corebio.seq.SeqList( alphabet = corebio.seq.dna_alphabet )
    for sequence in sequences:
        seqs.append( corebio.seq.dna( re.sub("\s", "-", sequence ) ) )

    data = weblogolib.LogoData.from_seqs(seqs)
    options = weblogolib.LogoOptions()
    options.color_scheme = weblogolib.classic

    options.title = 'A Logo Title'
    format = weblogolib.LogoFormat(data, options)
    fout = open(outfilename, 'w')
    weblogolib.png_formatter( data, format, fout)

if __name__ == "__main__":

    results = parse( sys.stdin)

    outname = "test%i.eps"

    import MAST
    for x, motifs in enumerate( results ):
        build_logo( [y.sequence for y in motifs.matches], outname % x )
        print motifs.score, motifs.sites

        outfile = open("test%i.motif" % x, "w")
        MAST.sequences2motif( outfile,
                              [y.sequence for y in motifs.matches] )
        outfile.close()

        outfile = open("test%i.tomtom" % x, "w")
        MAST.sequences2motif( outfile,
                              [y.sequence for y in motifs.matches],
                              format="TOMTOM")
        outfile.close()


    
